﻿using DataSharing.SendTask;
using DataSharing.Share.Enums;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using System.Net;
using XF.Domain.Dependency;
using XF.Domain.Repository;

namespace DataSharing.Province
{
    public class PusherProviderService : IPusherProviderService, IScopeDependency
    {
        private readonly ILogger<PusherProviderService> _logger;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly XieTongClient _xieTongClient;
        private readonly HuiJuClient _huiJuClient;
        private readonly IRepository<DsSendTask> _dsSendTaskRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="httpClientFactory"></param>
        /// <param name="xieTongClient"></param>
        /// <param name="huiJuClient"></param>
        /// <param name="dsSendTaskRepository"></param>
        public PusherProviderService(ILogger<PusherProviderService> logger,
            IChannelConfigurationManager channelConfigurationManager,
            IHttpClientFactory httpClientFactory,
            XieTongClient xieTongClient,
            HuiJuClient huiJuClient,
            IRepository<DsSendTask> dsSendTaskRepository)
        {
            _logger = logger;
            _channelConfigurationManager = channelConfigurationManager;
            _httpClientFactory = httpClientFactory;
            _xieTongClient = xieTongClient;
            _huiJuClient = huiJuClient;
            _dsSendTaskRepository = dsSendTaskRepository;
        }

        /// <summary>
        /// 省平台推送数据
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task SendProvinceDataPusher(DsSendTask dto, CancellationToken cancellationToken)
        {
            if (dto is null)
                return;

            ProvinceResponse result = new();

            if (dto.PathType == EPathType.HuiJu) //汇聚推送
                result = await _huiJuClient.RequestAsync<ProvinceResponse>(dto.Path, dto.HttpMethod, dto.Request, cancellationToken);

            else if (dto.PathType == EPathType.XieTong)//协同推送
                result = await _xieTongClient.RequestAsync<ProvinceResponse>(dto.Path, dto.HttpMethod, dto.Request, cancellationToken);

            else if (dto.PathType == EPathType.File)  //附件
            {
                if (dto != null && dto.FileJson != null && dto.FileJson.Count > 0)
                {
                    var itemFile = dto.FileJson[0];
                    result = await SendFileData(dto.Request, itemFile.FileId, itemFile.FileId, cancellationToken);
                }
            }

            if (result is not null)
            {
                //看看需不需要处理一些信息
                dto.LastTime = DateTime.Now;
                dto.SendTimes = dto.SendTimes++;

                if (dto.FirstTime is null)
                    dto.FirstTime = DateTime.Now;

                if (result.ReturnInfo is not null && result.ReturnInfo.Code == "1")
                    dto.IsSuccess = true;

                await _dsSendTaskRepository.UpdateAsync(dto, cancellationToken);
            }
        }

        /// <summary>
        /// 附件读取以及附件上传 
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="fileName"></param>
        /// <param name="fileId"></param>
        /// <returns></returns>
        private async Task<ProvinceResponse> SendFileData(string request, string fileName, string fileId, CancellationToken cancellationToken)
        {
            //获取配置信息
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            var businessFile = _channelConfigurationManager.GetConfigurationBusinessFile();
            byte[] fileContentBytes = null;
            try
            {
                string url = string.Format("{0}{1}?Source={2}&Id={3}", businessFile.BaseUrl, businessFile.DownloadUrlAddress, businessFile.Source, fileId);
                using (var client = _httpClientFactory.CreateClient())
                using (var response = await client.GetAsync(url))
                {
                    if (response.StatusCode == HttpStatusCode.OK)
                    {
                        using var memoryStream = new MemoryStream();
                        await response.Content.CopyToAsync(memoryStream);
                        memoryStream.Seek(0, SeekOrigin.Begin);
                        fileContentBytes = memoryStream.ToArray();
                    }
                }
                if (fileContentBytes != null)
                {
                    //获取附件上传Token
                    string strToken = await _xieTongClient.GetTokenAsync(cancellationToken);

                    Dictionary<string, object> dicParam = new()
                    {
                        { "params",  request } // 第一个接口参数，json格式字符串
                    };

                    // 构造字典文件数据
                    // 接口参数名称为files
                    CFormUpload.FileParameter fileParameter = new CFormUpload.FileParameter("files", fileContentBytes, fileName, null);
                    dicParam.Add(fileName, fileParameter);

                    string strUrl = configProvince.HuiJu + "get_case_material_info";
                    // 上传附件
                    string strResult = CFormUpload.MultipartFormDataPost(strUrl, null, dicParam, strToken);
                    if (!string.IsNullOrEmpty(strResult))
                        return JsonConvert.DeserializeObject<ProvinceResponse>(strResult);
                }
                return new ProvinceResponse();
            }
            catch (Exception)
            {
                return new ProvinceResponse();
            }
        }
    }
}
